library(shiny)
library(shinythemes)
library(DT)
library(markdown)
# Load libraries and functions needed to create SQLite databases.
library(RSQLite)
library(dplyr)
library(dbplyr)
library(lubridate)
source("ui.R")

### Save database

saveSQLite <- function(data, name){
  dir.create(file.path(paste(dirname(getwd()),"/",
                             basename(getwd()), sep=""), "basisdata"), showWarnings = FALSE)
  path <- paste(dirname(getwd()),"/",basename(getwd()),"/basisdata/",name,".sqlite", sep="")
  if (!file.exists(path)) {
    message("Menyimpan Basisdata pada direktori ", path)
    src <- src_sqlite(path, create = TRUE)
    copy_to(src, data, name, temporary = FALSE)
  } else {
    src <- src_sqlite(path)
    dbWriteTable(conn = src$con, name = name, 
                 value = data, row.names = FALSE, header = TRUE,
                 overwrite = T, append = F, temporary = FALSE)
    message("Overwrite Basisdata pada direktori ", path)
  }
  return (src)
}

### Read database
readSQLite <- function(name){
  path <- paste(dirname(getwd()),"/",basename(getwd()),"/basisdata/",name,".sqlite", sep="")
  if (!file.exists(path)) {
    message("Basisdata tidak ditemukan pada direktori ", path)
  } else {
    con <- dbConnect(SQLite(), path)
    df<- dbReadTable(con, name)
  }
  dbDisconnect(con)
  return (df)
}

### Get City Name
CityName <- function(abbr){
  as.data.frame(select(filter(kota_info, abbrev == abbr), name))[1,1]
}
##############################################
source("www/fungsi_agregasi.R",  local = TRUE)
initialtahun<-seq(2011, 2035, 1)
user_info <- readSQLite('membership')
kota_info <- readSQLite('kota')


server <- function(input, output,session) {
  
  ## Login-Logout
  #______________________________________________________________________________
  USER <- reactiveValues(Logged = FALSE, session = session$user)
  ROLE <- reactiveValues(Roled = 'user')
  City <- reactiveValues(Kota = "ALL")
  e_simpan <- reactiveValues(dfsave = readSQLite('df_e'))
  u_simpan <- reactiveValues(dfsave = readSQLite('df_u'))
  l_simpan <- reactiveValues(dfsave = readSQLite('df_l'))
  s_simpan <- reactiveValues(dfsave = readSQLite('df_s'))
  i_simpan <- reactiveValues(dfsave = readSQLite('df_i'))
  lengkap_simpan <- reactiveValues(dfsave = readSQLite('df_lengkap'))
  user_simpan <- reactiveValues(dfsave = readSQLite('membership'))
  ## Tabel Container
  source("www/tbl_container.R",  local = TRUE)
  #____________________________________________________________
  
  ## Ambil Role
  role <- reactive({
    curUser<-USER$name
    user_info <-readSQLite('membership')
    return(subset(user_info, (user == curUser))$role)
  })
  
  ## Ambil Kota/Kab
  kota_kab <- reactive({
    curUser<-USER$name
    user_info <-readSQLite('membership')
    return(CityName(subset(user_info, (user == curUser))$area))
  })
  
  ## Update Tabel Ekonomi
  df_e_baru<-reactive({
    df_e <- readSQLite('df_e')
    if(City$Kota =="ALL"){
      return(df_e)
    }else
      return(subset(df_e, (Kota.Kab == City$Kota)))
  })
  
  ## Update Tabel Keuangan
  df_u_baru<-reactive({
    df_u <- readSQLite('df_u')
    if(City$Kota =="ALL"){
      return(df_u)
    }else
      return(subset(df_u, (Kota.Kab == City$Kota)))
  })
  
  ## Update Tabel Sosial
  df_s_baru<-reactive({
    df_s <- readSQLite('df_s')
    if(City$Kota =="ALL"){
      return(df_s)
    }else
      return(subset(df_s, (Kota.Kab == City$Kota)))
  })
  
  ## Update Tabel Kelembagaan
  df_l_baru<-reactive({
    df_l <- readSQLite('df_l')
    if(City$Kota =="ALL"){
      return(df_l)
    }else
      return(subset(df_l, (Kota.Kab == City$Kota)))
  })
  
  ## Update Tabel Lingkungan
  df_i_baru<-reactive({
    df_i <- readSQLite('df_i')
    if(City$Kota =="ALL"){
      return(df_i)
    }else
      return(subset(df_i, (Kota.Kab == City$Kota)))
  })
  
  ## Update Tabel Lengkap
  df_lengkap_baru<-reactive({
    df_lengkap <- readSQLite('df_lengkap')
    if(City$Kota =="ALL"){
      return(df_lengkap)
    }else
      return(subset(df_lengkap, (Kota.Kab == City$Kota)))
  })
  
  ## Update Kota
  kota_pilih<-reactive({
    if(City$Kota =="ALL"){
      return(c('Kab. Bogor', 'Kab. Merauke'))
    }else{
      return(c(City$Kota))
    }
  })
  
  #_____________________________________________________________
  ###
  observe({
    if (USER$Logged == FALSE) {
      output$page <- renderUI({
        div(class="outer",
            do.call(bootstrapPage,c(titlePanel(
              fluidRow(column(12, img(height = 71, width = 709, src = "monfis-fem.png")))
              #fluidRow(column(9, "Fenologische modellen",offset = 20, style='padding:0px;'))
            ),ui1())))
      })
    }
   
    if ((USER$Logged == TRUE) & ROLE$Roled == 'user') # tambah catatan jika roleuser adalah guest --> Halaman Guest 
    {
      output$page <- renderUI({
        div(class="outer",
            do.call(bootstrapPage,c("",ui3()))) #"inverse=TRUE,title = "Anda Masuk di Halaman Bernada!
      })
    
    } else if ( (USER$Logged == TRUE) & ROLE$Roled == 'manager') {
      output$page <- renderUI({
        div(class="outer",
            do.call(bootstrapPage,c("",ui2())))
      })
    } else if ( (USER$Logged == TRUE) & ROLE$Roled == 'supadmin') {
      output$page <- renderUI({
        div(class="outer",
            do.call(bootstrapPage,c("",ui4())))
      })
    }
  })
  
  ## Kontrol Log In
  observeEvent(input$Login , {
    user_info <- readSQLite('membership')
    Username <- isolate(input$userName)
    Password <- isolate(input$passwd)
    Id.username <- which(any(user_info$user==Username))# cek ke sqlite
    Id.password <- which(any(user_info$pass==Password)) # cek ke sqilite
    if (length(Id.username) > 0 & length(Id.password) > 0) {
      if (Id.username == Id.password) {
        USER$Logged <- TRUE
        USER$name <- Username
        ROLE$Roled <- isolate(role())
        City$Kota <- isolate(kota_kab())
        values_e$dfWorking<-isolate(df_e_baru())
        values_u$dfWorking<-isolate(df_u_baru())
        values_l$dfWorking<-isolate(df_l_baru())
        values_s$dfWorking<-isolate(df_s_baru())
        values_i$dfWorking<-isolate(df_i_baru())
        values_lengkap$dfWorking<-isolate(df_lengkap_baru())
        values_user_info$dfWorking<- user_info
        updateSelectInput(session, "kota_e", choices = isolate(kota_pilih()))
        updateSelectInput(session, "kota_u", choices = isolate(kota_pilih()))
        updateSelectInput(session, "kota_l", choices = isolate(kota_pilih()))
        updateSelectInput(session, "kota_s", choices = isolate(kota_pilih()))
        updateSelectInput(session, "kota_i", choices = isolate(kota_pilih()))
        updateSelectInput(session, "kota_lengkap", choices = isolate(kota_pilih()))
      } 
    } else {
      USER$pass <- "User name atau password Salah!"
      showNotification(USER$pass)
    }
  })
  
  ## Kontrol Log out
  observeEvent(input$logout , {
        USER$Logged <- FALSE
    USER$pass <- ""
    aggg_result = -1
    if(aggg_result == -1)
    {
      session$reload()
      return()
      print("session reload not working")
    }
    })
  
  #______________________________________________________________________________
  
  ################################################################
  #                                                              #  
  #                    Indikator EKONOMI                         #
  #                                                              #  
  ################################################################
  
  ##
  output$tahun_e_pilih<- renderUI({
    selectInput("thn_e", "Tahun", as.list(initialtahun))
  })
  
  ##
  output$kota_e_pilih<- renderUI({
    selectInput("kota_e", "Kota/Kabupaten",
                choices = isolate(kota_pilih()))
  })
  
  ### Simpan Data
  observeEvent(input$save_e,{
    if(input$save_e > 0){
      df_e_new<-values_e$dfWorking
      df_e_old<-subset(readSQLite('df_e'), (Kota.Kab != City$Kota))
      if(City$Kota =="ALL"){
        e_simpan$dfsimpan <- df_e_new
      }else{
        e_simpan$dfsimpan<-merge(df_e_new, df_e_old, all = T)}
      #print(e_simpan$dfsimpan)
      saveSQLite(e_simpan$dfsimpan, "df_e")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data Indikator Ekonomi berhasil!"
      ))
    }})
  
  ### Update Nilai dan Menambah Baris
  observeEvent(input$addRows_e,{
    if(input$addRows_e > 0 & !is.na((as.numeric(input$thn_e)))){
      
      # untuk evaluasi Indikator
      #############################################################################
      n_e1<-indikator_2a(suppressWarnings(as.numeric(input$ind_e_1)), 0, 2, 5, 10)
      n_e2<-indikator_2a(suppressWarnings(as.numeric(input$ind_e_2)), 0, 2, 5, 10)
      n_e3<-indikator_2b(suppressWarnings(as.numeric(input$ind_e_3)), 20, 40, 60, 80)
      n_e4<-indikator_2b(suppressWarnings(as.numeric(input$ind_e_4)), 20, 40, 60, 80)
      n_e5<-indikator_2a(suppressWarnings(as.numeric(input$ind_e_5)), 0, 2, 5, 10)
      n_e6<-indikator_2b(suppressWarnings(as.numeric(input$ind_e_6)), 0, 1, 2.5, 5)
      n_e7<-indikator_2a(suppressWarnings(as.numeric(input$ind_e_7)), 0, 2, 5, 10)
      n_e8<-indikator_2a(suppressWarnings(as.numeric(input$ind_e_8)), 0, 2, 5, 10)
      nilai_e<-mean(c(n_e1, n_e2, n_e3, n_e4, n_e5, n_e6, n_e7, n_e8), na.rm = T)
      nilai_e <- format(round(nilai_e, 2), nsmall = 2)
      cond_e <- kondisi(suppressWarnings(as.numeric(nilai_e)))
      
      #############################################################################
      temp_df<-isolate(c(as.character(input$thn_e), as.character(input$kota_e), 
                         as.character(input$ind_e_1), as.character(kondisi(suppressWarnings(as.numeric(n_e1)))),
                         as.character(input$ind_e_2), as.character(kondisi(suppressWarnings(as.numeric(n_e2)))),
                         as.character(input$ind_e_3), as.character(kondisi(suppressWarnings(as.numeric(n_e3)))), 
                         as.character(input$ind_e_4), as.character(kondisi(suppressWarnings(as.numeric(n_e4)))),
                         as.character(input$ind_e_5), as.character(kondisi(suppressWarnings(as.numeric(n_e5)))),
                         as.character(input$ind_e_6), as.character(kondisi(suppressWarnings(as.numeric(n_e6)))),
                         as.character(input$ind_e_7), as.character(kondisi(suppressWarnings(as.numeric(n_e7)))),
                         as.character(input$ind_e_8), as.character(kondisi(suppressWarnings(as.numeric(n_e8)))),
                         as.character(nilai_e),as.character(cond_e)))
      isolate(values_e$dfWorking <- as.data.frame(lapply(rbind(values_e$dfWorking,temp_df), as.character),stringsAsFactors=FALSE))
      isolate(names(values_e$dfWorking)<-c('Tahun', 'Kota.Kab', 
                                           'Nilai.E1','Kondisi.E1', 
                                           'Nilai.E2','Kondisi.E2', 
                                           'Nilai.E3','Kondisi.E3',
                                           'Nilai.E4','Kondisi.E4', 
                                           'Nilai.E5','Kondisi.E5', 
                                           'Nilai.E6','Kondisi.E6', 
                                           'Nilai.E7','Kondisi.E7',
                                           'Nilai.E8','Kondisi.E8', 
                                           'Nilai.Total', 'Kondisi.Total'))
      #saveSQLite(values_e$dfWorking, "df_e")
      remove_e <-as.numeric({values_e$dfWorking[,1]})
      tahun_e_eliminate<- initialtahun[! initialtahun %in% remove_e]
      updateSelectInput(session, "thn_e", choices = tahun_e_eliminate)
    }
  })
  
  ### Menghapus Baris
  observeEvent(input$deleteRows_e,{
    if (!is.null(input$table_e_rows_selected)) {
      values_e$dfWorking <- values_e$dfWorking[-as.numeric(input$table_e_rows_selected),]
      #saveSQLite(values_e$dfWorking, "df_e")
      update_e <-as.numeric({values_e$dfWorking[,1]})
      tahun_e_update<- initialtahun[! initialtahun %in% update_e]
      updateSelectInput(session, "thn_e", choices = tahun_e_update)
    }
    else{
      values_e$dfWorking <- values_e$dfWorking[-as.numeric(nrow(values_e$dfWorking)),]
      #saveSQLite(values_e$dfWorking, "df_e")
      update_e <-as.numeric({values_e$dfWorking[,1]})
      tahun_e_update<- initialtahun[! initialtahun %in% update_e]
      updateSelectInput(session, "thn_e", choices = tahun_e_update)
    }
  })
  ### Mendownload File
  output$download_e <- downloadHandler(
    filename = function(){paste(format(Sys.time(), "%Y_%m_%d_"),"Indikator Ekonomi.csv", sep ="")}, 
    content = function(fname){
      file_e_unduh <- {values_e$dfWorking[order(values_e$dfWorking$Tahun, decreasing=F), ]}
      names(file_e_unduh) <- c('Tahun', 'Kota/Kab', 
                               'Nilai E1', 'Kondisi E1',
                               'Nilai E2','Kondisi E2', 
                               'Nilai E3','Kondisi E3', 
                               'Nilai E4','Kondisi E4',
                               'Nilai E5','Kondisi E5', 
                               'Nilai E6','Kondisi E6', 
                               'Nilai E7','Kondisi E7', 
                               'Nilai E8','Kondisi E8', 
                               'Nilai Total', 'Kondisi Total')
      write.csv(file_e_unduh, fname, row.names = F)
      
    }
  )
  
  ### Untuk Upload Data
  filedata_e <- reactive({
    infile_e <- input$fileInput_e
    if (is.null(infile_e)) {
      # User has not uploaded a file yet
      return(NULL)
    }
    read.csv(infile_e$datapath, header=T, sep=",")
  })
  
  # Load File CSV untuk indikator Ekonomi
  observeEvent(input$shows_e,{
    if(!(is.null({filedata_e()}))){
      temp_e <- data.frame(isolate(filedata_e()))
      id_name_e<-c('Tahun', 'Kota.Kab', 
                   'Nilai.E1','Kondisi.E1', 
                   'Nilai.E2','Kondisi.E2', 
                   'Nilai.E3','Kondisi.E3',
                   'Nilai.E4','Kondisi.E4', 
                   'Nilai.E5','Kondisi.E5', 
                   'Nilai.E6','Kondisi.E6', 
                   'Nilai.E7','Kondisi.E7',
                   'Nilai.E8','Kondisi.E8', 
                   'Nilai.Total', 'Kondisi.Total')
      if(identical(id_name_e, names(temp_e))){
        ####
        kon_e1<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[3])), 0, 2, 5, 10))))
        new_df <- cbind(temp_e[1:nrow(temp_e), 1:3], "Kondisi.E1"=kon_e1, "Nilai.E2" = temp_e[1:nrow(temp_e), 5])
        kon_e2<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[5])), 0, 2, 5, 10))))
        new_df <- cbind(new_df, "Kondisi.E2"=kon_e2, "Nilai.E3"=temp_e[1:nrow(temp_e), 7])
        kon_e3<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2b(suppressWarnings(as.numeric(x[7])), 20, 40, 60, 80))))
        new_df <- cbind(new_df, "Kondisi.E3"=kon_e3, "Nilai.E4"=temp_e[1:nrow(temp_e), 9])
        kon_e4<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2b(suppressWarnings(as.numeric(x[9])), 20, 40, 60, 80))))
        new_df <- cbind(new_df, "Kondisi.E4"=kon_e4, "Nilai.E5"=temp_e[1:nrow(temp_e), 11])
        kon_e5<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[11])), 0, 2, 5, 10))))
        new_df <- cbind(new_df, "Kondisi.E5"=kon_e5, "Nilai.E6"=temp_e[1:nrow(temp_e), 13])
        kon_e6<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2b(suppressWarnings(as.numeric(x[13])), 0, 1, 2.5, 5))))
        new_df <- cbind(new_df, "Kondisi.E6"=kon_e6, "Nilai.E7"=temp_e[1:nrow(temp_e), 15])
        kon_e7<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[15])), 0, 2, 5, 10))))
        new_df <- cbind(new_df, "Kondisi.E7"=kon_e7, "Nilai.E8"=temp_e[1:nrow(temp_e), 17])
        kon_e8<-apply(temp_e, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[17])), 0, 2, 5, 10))))
        new_df <- cbind(new_df, "Kondisi.E8"=kon_e8)
        ####
        result1 <- apply(temp_e, 1, 
                         function(x) mean(c(
                           indikator_2a(suppressWarnings(as.numeric(x[3])), 0, 2, 5, 10),
                           indikator_2a(suppressWarnings(as.numeric(x[5])), 0, 2, 5, 10),
                           indikator_2b(suppressWarnings(as.numeric(x[7])), 20, 40, 60, 80),
                           indikator_2b(suppressWarnings(as.numeric(x[9])), 20, 40, 60, 80),
                           indikator_2a(suppressWarnings(as.numeric(x[11])), 0, 2, 5, 10),
                           indikator_2b(suppressWarnings(as.numeric(x[13])), 0, 1, 2.5, 5),
                           indikator_2a(suppressWarnings(as.numeric(x[15])), 0, 2, 5, 10),
                           indikator_2a(suppressWarnings(as.numeric(x[17])), 0, 2, 5, 10)), na.rm = T
                         ))
        result1<-format(round(result1, 2), nsmall = 2)
        new_df <- cbind(new_df, "Nilai.Total"=result1)
        result2 <- apply(new_df, 1, function(x) kondisi(as.numeric(x[19])))
        new_df2 <- cbind(new_df, "Kondisi.Total"=result2)
        temp_e <- new_df2
        ####
        if((input$shows_e > 0)){
          values_e$dfWorking <- as.data.frame(rbind(values_e$dfWorking,temp_e), stringsAsFactors=FALSE)
          values_e$dfWorking[] <- lapply(values_e$dfWorking, as.character)
          update_e <-as.numeric({values_e$dfWorking[,1]})
          tahun_e_update<- initialtahun[! initialtahun %in% update_e]
          updateSelectInput(session, "thn_e", choices = tahun_e_update)
          temp_e<-NULL
        }else return (NULL)
      }
    }
  }
  )
  
  ################################################################
  #                                                              #
  #                    Indikator KEUANGAN                        #
  #                                                              #  
  ################################################################  
  
  ##
  output$tahun_u_pilih<- renderUI({
    selectInput("thn_u", "Tahun", as.list(initialtahun))
  })
  
  ##
  output$kota_u_pilih<- renderUI({
    selectInput("kota_u", "Kota/Kabupaten",
                choices = isolate(kota_pilih()))
  })
  
  ### Simpan Data
  observeEvent(input$save_u,{
    if(input$save_u > 0){
      df_u_new<-values_u$dfWorking
      df_u_old<-subset(readSQLite('df_u'), (Kota.Kab != City$Kota))
      if(City$Kota =="ALL"){
        u_simpan$dfsimpan <- df_u_new
      }else{
        u_simpan$dfsimpan<-merge(df_u_new, df_u_old, all = T)}
      #print(u_simpan$dfsimpan)
      saveSQLite(u_simpan$dfsimpan, "df_u")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data Indikator Keuangan berhasil!"
      ))
    }})
  
  
  ### Update Nilai dan Menambah Baris
  observeEvent(input$addRows_u,{
    if(input$addRows_u > 0 & !is.na((as.numeric(input$thn_u)))){
      
      # untuk evaluasi Indikator
      #############################################################################
      n_u1<-indikator_2a(suppressWarnings(as.numeric(input$ind_u_1)), 20, 40, 60, 80)
      n_u2<-indikator_2b(suppressWarnings(as.numeric(input$ind_u_2)), 15, 30, 50, 80)
      n_u3<-indikator_2b(suppressWarnings(as.numeric(input$ind_u_3)), 15, 30, 50, 80)
      nilai_u<-mean(c(n_u1, n_u2, n_u3), na.rm = T)
      nilai_u <- format(round(nilai_u, 2), nsmall = 2)
      cond_u <- kondisi(suppressWarnings(as.numeric(nilai_u)))
      
      #############################################################################
      temp_df<-isolate(c(as.character(input$thn_u), 
                         as.character(input$kota_u), 
                         as.character(input$ind_u_1),as.character(kondisi(suppressWarnings(as.numeric(n_u1)))), 
                         as.character(input$ind_u_2), as.character(kondisi(suppressWarnings(as.numeric(n_u2)))),
                         as.character(input$ind_u_3), as.character(kondisi(suppressWarnings(as.numeric(n_u3)))),
                         as.character(nilai_u),
                         as.character(cond_u)))
      isolate(values_u$dfWorking <- as.data.frame(lapply(rbind(values_u$dfWorking,temp_df), as.character),stringsAsFactors=FALSE))
      isolate(names(values_u$dfWorking)<-c('Tahun', 
                                           'Kota.Kab', 
                                           'Nilai.U1', 'Kondisi.U1', 
                                           'Nilai.U2', 'Kondisi.U2',
                                           'Nilai.U3', 'Kondisi.U3',
                                           'Nilai.Total', 'Kondisi.Total')
      )
      #saveSQLite(values_u$dfWorking, "df_u")
      remove_u <-as.numeric({values_u$dfWorking[,1]})
      tahun_u_eliminate<- initialtahun[! initialtahun %in% remove_u]
      updateSelectInput(session, "thn_u", choices = tahun_u_eliminate)
    }
  })
  
  ### Menghapus Baris
  observeEvent(input$deleteRows_u,{
    if (!is.null(input$table_u_rows_selected)) {
      values_u$dfWorking <- values_u$dfWorking[-as.numeric(input$table_u_rows_selected),]
      #saveSQLite(values_u$dfWorking, "df_u")
      update_u <-as.numeric({values_u$dfWorking[,1]})
      tahun_u_update<- initialtahun[! initialtahun %in% update_u]
      updateSelectInput(session, "thn_u", choices = tahun_u_update)
    }
    else{
      values_u$dfWorking <- values_u$dfWorking[-as.numeric(nrow(values_u$dfWorking)),]
      #saveSQLite(values_u$dfWorking, "df_u")
      update_u <-as.numeric({values_u$dfWorking[,1]})
      tahun_u_update<- initialtahun[! initialtahun %in% update_u]
      updateSelectInput(session, "thn_u", choices = tahun_u_update)
    }
  })
  ### Mendownload File
  output$download_u <- downloadHandler(
    filename = function(){paste(format(Sys.time(), "%Y_%m_%d_"),"Indikator Keuangan.csv", sep ="")}, 
    content = function(fname){
      file_u_unduh <- {values_u$dfWorking[order(values_u$dfWorking$Tahun, decreasing=F), ]}
      names(file_u_unduh) <- c('Tahun', 
                               'Kota/Kab', 
                               'Nilai U1', 'Kondisi U1', 
                               'Nilai U2', 'Kondisi U2',
                               'Nilai U3', 'Kondisi U3',
                               'Nilai Total', 'Kondisi Total')
      write.csv(file_u_unduh, fname, row.names = F)
    }
  )
  
  ### Untuk Upload Data
  filedata_u <- reactive({
    infile_u <- input$fileInput_u
    if (is.null(infile_u)) {
      # User has not uploaded a file yet
      return(NULL)
    }
    read.csv(infile_u$datapath, header=T, sep=",")
  })
  
  # Load File CSV untuk indikator Keuangan
  observeEvent(input$shows_u,{
    if(!(is.null({filedata_u()}))){
      temp_u <- data.frame(isolate(filedata_u()))
      id_name_u <- c('Tahun', 
                     'Kota.Kab', 
                     'Nilai.U1', 'Kondisi.U1', 
                     'Nilai.U2', 'Kondisi.U2',
                     'Nilai.U3', 'Kondisi.U3',
                     'Nilai.Total', 'Kondisi.Total')
      if(identical(id_name_u, names(temp_u))){
        kon_u1<-apply(temp_u, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[3])), 20, 40, 60, 80))))
        new_df <- cbind(temp_u[1:nrow(temp_u), 1:3], "Kondisi.U1"=kon_u1, "Nilai.U2" = temp_u[1:nrow(temp_u), 5])
        kon_u2<-apply(temp_u, 1, function(x) kondisi(as.numeric(indikator_2b(suppressWarnings(as.numeric(x[5])), 15, 30, 50, 80))))
        new_df <- cbind(new_df, "Kondisi.U2"=kon_u2, "Nilai.U3"=temp_u[1:nrow(temp_u), 7])
        kon_u3<-apply(temp_u, 1, function(x) kondisi(as.numeric(indikator_2b(suppressWarnings(as.numeric(x[7])), 15, 30, 50, 80))))
        new_df <- cbind(new_df, "Kondisi.U3"=kon_u3)
        ####
        result1 <- apply(temp_u, 1, 
                         function(x) mean(c(
                           indikator_2a(suppressWarnings(as.numeric(x[3])), 20, 40, 60, 80),
                           indikator_2b(suppressWarnings(as.numeric(x[5])),  15, 30, 50, 80),
                           indikator_2b(suppressWarnings(as.numeric(x[7])), 15, 30, 50, 80)), na.rm = T
                         ))
        result1<-format(round(result1, 2), nsmall = 2)
        new_df <- cbind(new_df, "Nilai.Total"=result1)
        result2 <- apply(new_df, 1, function(x) kondisi(as.numeric(x[9])))
        new_df2 <- cbind(new_df, "Kondisi.Total"=result2)
        temp_u <- new_df2
        ####
        if(input$shows_u > 0){
          values_u$dfWorking <- as.data.frame(rbind(values_u$dfWorking,temp_u), stringsAsFactors=FALSE)
          values_u$dfWorking[] <- lapply(values_u$dfWorking, as.character)
          update_u <-as.numeric({values_u$dfWorking[,1]})
          tahun_u_update<- initialtahun[! initialtahun %in% update_u]
          updateSelectInput(session, "thn_u", choices = tahun_u_update) 
          temp_u <- NULL
        }else return (NULL)
      }
      
    }
  })
  
  ################################################################
  #                                                              #
  #                    Indikator Kelembagaan                     #
  #                                                              #  
  ################################################################  
  
  ##
  output$tahun_l_pilih<- renderUI({
    selectInput("thn_l", "Tahun", as.list(initialtahun))
  })
  
  ##
  output$kota_l_pilih<- renderUI({
    selectInput("kota_l", "Kota/Kabupaten",
                choices = isolate(kota_pilih()))
  })
  
  ### Simpan Data
  observeEvent(input$save_l,{
    if(input$save_l > 0){
      df_l_new<-values_l$dfWorking
      df_l_old<-subset(readSQLite('df_l'), (Kota.Kab != City$Kota))
      if(City$Kota =="ALL"){
        l_simpan$dfsimpan <- df_l_new
      }else{
        l_simpan$dfsimpan<-merge(df_l_new, df_l_old, all = T)}
      #print(l_simpan$dfsimpan)
      saveSQLite(l_simpan$dfsimpan, "df_l")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data Indikator Kelembagaan berhasil!"
      ))
      saveSQLite(values_l$dfWorking, "df_l")
    }})
  
  ### Update Nilai dan Menambah Baris
  observeEvent(input$addRows_l,{
    if(input$addRows_l > 0& !is.na((as.numeric(input$thn_l)))){
      
      # untuk evaluasi Indikator
      #############################################################################
      n_l1<-indikator_2a(suppressWarnings(as.numeric(input$ind_l_1)), 1, 2, 5, 10)
      n_l2<-indikator_2a(suppressWarnings(as.numeric(input$ind_l_2)), 10, 20, 40, 60)
      n_l3<-indikator_2a(suppressWarnings(as.numeric(input$ind_l_3)), 1, 2, 5, 10)
      nilai_l<-mean(c(n_l1, n_l2, n_l3), na.rm = T)
      nilai_l <- format(round(nilai_l, 2), nsmall = 2)
      cond_l <- kondisi(suppressWarnings(as.numeric(nilai_l)))
      
      #############################################################################
      temp_df<-isolate(c(as.character(input$thn_l),
                         as.character(input$kota_l), 
                         as.character(input$ind_l_1), as.character(kondisi(suppressWarnings(as.numeric(n_l1)))), 
                         as.character(input$ind_l_2), as.character(kondisi(suppressWarnings(as.numeric(n_l2)))),
                         as.character(input$ind_l_3), as.character(kondisi(suppressWarnings(as.numeric(n_l3)))),
                         as.character(nilai_l),
                         as.character(cond_l)))
      
      isolate(values_l$dfWorking <- as.data.frame(lapply(rbind(values_l$dfWorking,temp_df), as.character),stringsAsFactors=FALSE))
      isolate(names(values_l$dfWorking)<-c('Tahun', 'Kota.Kab',
                                           'Nilai.L1', 'Kondisi.L1', 
                                           'Nilai.L2', 'Kondisi.L2',
                                           'Nilai.L3', 'Kondisi.L3',
                                           'Nilai.Total', 
                                           'Kondisi.Total'))
      #saveSQLite(values_l$dfWorking, "df_l")
      remove_l <-as.numeric({values_l$dfWorking[,1]})
      tahun_l_eliminate<- initialtahun[! initialtahun %in% remove_l]
      updateSelectInput(session, "thn_l", choices = tahun_l_eliminate)
    }
  })
  
  ### Menghapus Baris
  observeEvent(input$deleteRows_l,{
    if (!is.null(input$table_l_rows_selected)) {
      values_l$dfWorking <- values_l$dfWorking[-as.numeric(input$table_l_rows_selected),]
      #saveSQLite(values_l$dfWorking, "df_l")
      update_l <-as.numeric({values_l$dfWorking[,1]})
      tahun_l_update<- initialtahun[! initialtahun %in% update_l]
      updateSelectInput(session, "thn_l", choices = tahun_l_update)
    }
    else{
      values_l$dfWorking <- values_l$dfWorking[-as.numeric(nrow(values_l$dfWorking)),]
      #saveSQLite(values_l$dfWorking, "df_l")
      update_l <-as.numeric({values_l$dfWorking[,1]})
      tahun_l_update<- initialtahun[! initialtahun %in% update_l]
      updateSelectInput(session, "thn_l", choices = tahun_l_update)
    }
  })
  ### Mendownload File
  output$download_l <- downloadHandler(
    filename = function(){paste(format(Sys.time(), "%Y_%m_%d_"),"Indikator Kelembagaan.csv", sep ="")}, 
    content = function(fname){
      file_l_unduh <- {values_l$dfWorking[order(values_l$dfWorking$Tahun, decreasing=F), ]}
      names(file_l_unduh) <- c('Tahun', 'Kota/Kab',
                               'Nilai L1', 'Kondisi L1', 
                               'Nilai L2', 'Kondisi L2',
                               'Nilai L3', 'Kondisi L3',
                               'Nilai Total', 
                               'Kondisi Total')
      write.csv(file_l_unduh, fname, row.names = F)
    }
  )
  
  ### Untuk Upload Data
  filedata_l <- reactive({
    infile_l <- input$fileInput_l
    if (is.null(infile_l)) {
      # User has not uploaded a file yet
      return(NULL)
    }
    read.csv(infile_l$datapath, header=T, sep=",")
  })
  
  # Load File CSV untuk indikator Kelembagaan
  observeEvent(input$shows_l,{
    if(!(is.null({filedata_l()}))){
      temp_l <- data.frame(isolate(filedata_l()))
      id_name_l<- c('Tahun', 'Kota.Kab',
                    'Nilai.L1', 'Kondisi.L1', 
                    'Nilai.L2', 'Kondisi.L2',
                    'Nilai.L3', 'Kondisi.L3',
                    'Nilai.Total', 
                    'Kondisi.Total')
      if(identical(id_name_l, names(temp_l))){
        kon_l1<-apply(temp_l, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[3])), 1, 2, 5, 10))))
        new_df <- cbind(temp_l[1:nrow(temp_l), 1:3], "Kondisi.L1"=kon_l1, "Nilai.L2" = temp_l[1:nrow(temp_l), 5])
        kon_l2<-apply(temp_l, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[5])), 10, 20, 40, 60))))
        new_df <- cbind(new_df, "Kondisi.L2"=kon_l2, "Nilai.L3"=temp_l[1:nrow(temp_l), 7])
        kon_l3<-apply(temp_l, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[7])), 1, 2, 5, 10))))
        new_df <- cbind(new_df, "Kondisi.L3"=kon_l3)
        ####
        ####
        result1 <- apply(temp_l, 1, 
                         function(x) mean(c(
                           indikator_2a(suppressWarnings(as.numeric(x[3])), 1, 2, 5, 10),
                           indikator_2a(suppressWarnings(as.numeric(x[5])), 10, 20, 40, 60),
                           indikator_2a(suppressWarnings(as.numeric(x[7])), 1, 2, 5, 10)), na.rm = T
                         ))
        result1<-format(round(result1, 2), nsmall = 2)
        new_df <- cbind(new_df, "Nilai.Total"=result1)
        result2 <- apply(new_df, 1, function(x) kondisi(as.numeric(x[9])))
        new_df2 <- cbind(new_df, "Kondisi.Total"=result2)
        temp_l <- new_df2
        ###
        if(input$shows_l > 0){
          values_l$dfWorking <- rbind(values_l$dfWorking, temp_l)
          values_l$dfWorking[] <- lapply(values_l$dfWorking, as.character)
          update_l <-as.numeric({values_l$dfWorking[,1]})
          tahun_l_update<- initialtahun[! initialtahun %in% update_l]
          updateSelectInput(session, "thn_l", choices = tahun_l_update) 
          temp_l<- NULL
        }else return (NULL)
      }
    }
  }
  )
  
  ################################################################
  #                                                              #
  #                    Indikator Sosial                          #
  #                                                              #  
  ################################################################  
  
  ##
  output$tahun_s_pilih<- renderUI({
    selectInput("thn_s", "Tahun", as.list(initialtahun))
  })
  
  ##
  output$kota_s_pilih<- renderUI({
    selectInput("kota_s", "Kota/Kabupaten",
                choices = isolate(kota_pilih()))
  })
  
  ### Simpan Data
  observeEvent(input$save_s,{
    if(input$save_s > 0){
      df_s_new<-values_s$dfWorking
      df_s_old<-subset(readSQLite('df_s'), (Kota.Kab != City$Kota))
      if(City$Kota =="ALL"){
        s_simpan$dfsimpan <- df_s_new
      }else{
        s_simpan$dfsimpan<-merge(df_s_new, df_s_old, all = T)}
      #print(s_simpan$dfsimpan)
      saveSQLite(s_simpan$dfsimpan, "df_s")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data Indikator Sosial berhasil!"
      ))
      saveSQLite(values_s$dfWorking, "df_s")
    }})
  
  
  ### Update Nilai dan Menambah Baris
  observeEvent(input$addRows_s,{
    if(input$addRows_s > 0& !is.na((as.numeric(input$thn_s)))){
      
      # untuk evaluasi Indikator
      #############################################################################
      n_s1<-indikator_1b(suppressWarnings(as.numeric(input$ind_s_1)), 10, 15, 30, 60)
      n_s2<-indikator_1b(suppressWarnings(as.numeric(input$ind_s_2)), 10, 15, 30, 60)
      n_s3<-indikator_1b(suppressWarnings(as.numeric(input$ind_s_3)), 10, 15, 30, 60)
      n_s4<-indikator_1b(suppressWarnings(as.numeric(input$ind_s_4)), 10, 15, 30, 60)
      n_s5<-indikator_1b(suppressWarnings(as.numeric(input$ind_s_5)), 10, 15, 30, 60)
      
      nilai_s<-mean(c(n_s1, n_s2, n_s3, n_s4, n_s5), na.rm = T)
      nilai_s <- format(round(nilai_s, 2), nsmall = 2)
      
      cond_s <- kondisi(suppressWarnings(as.numeric(nilai_s)))
      
      #############################################################################
      temp_df<-isolate(c(as.character(input$thn_s), as.character(input$kota_s), 
                         as.character(input$ind_s_1), as.character(kondisi(suppressWarnings(as.numeric(n_s1)))), 
                         as.character(input$ind_s_2), as.character(kondisi(suppressWarnings(as.numeric(n_s2)))),
                         as.character(input$ind_s_3), as.character(kondisi(suppressWarnings(as.numeric(n_s3)))),
                         as.character(input$ind_s_4), as.character(kondisi(suppressWarnings(as.numeric(n_s4)))),
                         as.character(input$ind_s_5), as.character(kondisi(suppressWarnings(as.numeric(n_s5)))),
                         as.character(nilai_s),as.character(cond_s)))
      isolate(values_s$dfWorking <- as.data.frame(lapply(rbind(values_s$dfWorking,temp_df), as.character),stringsAsFactors=FALSE))
      isolate(names(values_s$dfWorking)<-c('Tahun', 'Kota.Kab', 
                                           'Nilai.S1', 'Kondisi.S1',
                                           'Nilai.S2', 'Kondisi.S2',
                                           'Nilai.S3', 'Kondisi.S3',
                                           'Nilai.S4', 'Kondisi.S4',
                                           'Nilai.S5', 'Kondisi.S5',
                                           'Nilai.Total', 'Kondisi.Total'))
      #saveSQLite(values_s$dfWorking, "df_s")
      remove_s <-as.numeric({values_s$dfWorking[,1]})
      tahun_s_eliminate<- initialtahun[! initialtahun %in% remove_s]
      updateSelectInput(session, "thn_s", choices = tahun_s_eliminate)
    }
  })
  
  ### Menghapus Baris
  observeEvent(input$deleteRows_s,{
    if (!is.null(input$table_s_rows_selected)) {
      values_s$dfWorking <- values_s$dfWorking[-as.numeric(input$table_s_rows_selected),]
      #saveSQLite(values_s$dfWorking, "df_s")
      update_s <-as.numeric({values_s$dfWorking[,1]})
      tahun_s_update<- initialtahun[! initialtahun %in% update_s]
      updateSelectInput(session, "thn_s", choices = tahun_s_update)
    }
    else{
      values_s$dfWorking <- values_s$dfWorking[-as.numeric(nrow(values_s$dfWorking)),]
      #saveSQLite(values_s$dfWorking, "df_s")
      update_s <-as.numeric({values_s$dfWorking[,1]})
      tahun_s_update<- initialtahun[! initialtahun %in% update_s]
      updateSelectInput(session, "thn_s", choices = tahun_s_update)
    }
  })
  ### Mendownload File
  output$download_s <- downloadHandler(
    filename = function(){paste(format(Sys.time(), "%Y_%m_%d_"),"Indikator Sosial.csv", sep ="")}, 
    content = function(fname){
      file_s_unduh <- {values_s$dfWorking[order(values_s$dfWorking$Tahun, decreasing=F), ]}
      names(file_s_unduh) <- c('Tahun', 'Kota/Kab', 
                               'Nilai S1', 'Kondisi S1',
                               'Nilai S2', 'Kondisi S2',
                               'Nilai S3', 'Kondisi S3',
                               'Nilai S4', 'Kondisi S4',
                               'Nilai S5', 'Kondisi S5',
                               'Nilai Total', 'Kondisi Total')
      write.csv(file_s_unduh, fname, row.names = F)
    }
  )
  
  ### Untuk Upload Data
  filedata_s <- reactive({
    infile_s <- input$fileInput_s
    if (is.null(infile_s)) {
      # User has not uploaded a file yet
      return(NULL)
    }
    read.csv(infile_s$datapath, header=T, sep=",")
  })
  
  # Load File CSV untuk indikator Sosial
  observeEvent(input$shows_s,{
    if(!(is.null({filedata_s()}))){
      temp_s <- data.frame(isolate(filedata_s()))
      id_name_s<-c('Tahun', 'Kota.Kab', 
                   'Nilai.S1', 'Kondisi.S1',
                   'Nilai.S2', 'Kondisi.S2',
                   'Nilai.S3', 'Kondisi.S3',
                   'Nilai.S4', 'Kondisi.S4',
                   'Nilai.S5', 'Kondisi.S5',
                   'Nilai.Total', 'Kondisi.Total')
      ####
      if(identical(id_name_s, names(temp_s))){
        ####
        kon_s1<-apply(temp_s, 1, function(x) kondisi(as.numeric(indikator_1b(suppressWarnings(as.numeric(x[3])), 10, 15, 30, 60))))
        new_df <- cbind(temp_s[1:nrow(temp_s), 1:3], "Kondisi.S1"=kon_s1, "Nilai.S2" = temp_s[1:nrow(temp_s), 5])
        kon_s2<-apply(temp_s, 1, function(x) kondisi(as.numeric(indikator_1b(suppressWarnings(as.numeric(x[5])), 10, 15, 30, 60))))
        new_df <- cbind(new_df, "Kondisi.S2"=kon_s2, "Nilai.S3"=temp_s[1:nrow(temp_s), 7])
        kon_s3<-apply(temp_s, 1, function(x) kondisi(as.numeric(indikator_1b(suppressWarnings(as.numeric(x[7])), 10, 15, 30, 60))))
        new_df <- cbind(new_df, "Kondisi.S3"=kon_s3, "Nilai.S4"=temp_s[1:nrow(temp_s), 9])
        kon_s4<-apply(temp_s, 1, function(x) kondisi(as.numeric(indikator_1b(suppressWarnings(as.numeric(x[9])), 10, 15, 30, 60))))
        new_df <- cbind(new_df, "Kondisi.S4"=kon_s4, "Nilai.S5"=temp_s[1:nrow(temp_s), 11])
        kon_s5<-apply(temp_s, 1, function(x) kondisi(as.numeric(indikator_1b(suppressWarnings(as.numeric(x[11])), 10, 15, 30, 60))))
        new_df <- cbind(new_df, "Kondisi.S5"=kon_s5)
        ####
        result1 <- apply(temp_s, 1, 
                         function(x) mean(c(
                           indikator_1b(suppressWarnings(as.numeric(x[3])), 10, 15, 30, 60),
                           indikator_1b(suppressWarnings(as.numeric(x[5])), 10, 15, 30, 60),
                           indikator_1b(suppressWarnings(as.numeric(x[7])), 10, 15, 30, 60),
                           indikator_1b(suppressWarnings(as.numeric(x[9])), 10, 15, 30, 60),
                           indikator_1b(suppressWarnings(as.numeric(x[11])), 10, 15, 30, 60)), na.rm = T
                         ))
        result1<-format(round(result1, 2), nsmall = 2)
        new_df <- cbind(new_df, "Nilai.Total"=result1)
        result2 <- apply(new_df, 1, function(x) kondisi(as.numeric(x[13])))
        new_df2 <- cbind(new_df, "Kondisi.Total"=result2)
        temp_s <- new_df2
        if(input$shows_s > 0){
          values_s$dfWorking <- as.data.frame(rbind(values_s$dfWorking,temp_s), stringsAsFactors=FALSE)
          values_s$dfWorking[] <- lapply(values_s$dfWorking, as.character)
          update_s <-as.numeric({values_s$dfWorking[,1]})
          tahun_s_update<- initialtahun[! initialtahun %in% update_s]
          updateSelectInput(session, "thn_s", choices = tahun_s_update) 
          temp_s<-NULL
        }else return (NULL) 
      }
    }
  }
  )
  
  ################################################################
  #                                                              #
  #                    Indikator Lingkungan                      #
  #                                                              #  
  ################################################################  
  
  ##
  output$tahun_i_pilih<- renderUI({
    selectInput("thn_i", "Tahun", as.list(initialtahun))
  })
  
  ##
  output$kota_i_pilih<- renderUI({
    selectInput("kota_i", "Kota/Kabupaten",
                choices = isolate(kota_pilih()))
  })
  
  ### Simpan Data
  observeEvent(input$save_i,{
    if(input$save_i > 0){
      df_i_new<-values_i$dfWorking
      df_i_old<-subset(readSQLite('df_i'), (Kota.Kab != City$Kota))
      if(City$Kota =="ALL"){
        i_simpan$dfsimpan <- df_i_new
      }else{
        i_simpan$dfsimpan<-merge(df_i_new, df_i_old, all = T)}
      #print(i_simpan$dfsimpan)
      saveSQLite(i_simpan$dfsimpan, "df_i")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data Indikator Lingkungan berhasil!"
      ))
      saveSQLite(values_i$dfWorking, "df_i")
    }})
  
  ### Update Nilai dan Menambah Baris
  observeEvent(input$addRows_i,{
    if(input$addRows_i > 0 & !is.na((as.numeric(input$thn_i)))){
      # untuk evaluasi Indikator
      #############################################################################
      n_i1<-indikator_2a(suppressWarnings(as.numeric(input$ind_i_1)), 0.5, 1, 3, 5)
      n_i2<-indikator_2a(suppressWarnings(as.numeric(input$ind_i_2)), 0.5, 1, 3, 5)
      nilai_i<-mean(c(n_i1, n_i2), na.rm = T)
      nilai_i <- format(round(nilai_i, 2), nsmall = 2)
      cond_i <- kondisi(suppressWarnings(as.numeric(nilai_i)))
      #############################################################################
      temp_df<-isolate(c(as.character(input$thn_i), 
                         as.character(input$kota_i), 
                         as.character(input$ind_i_1), as.character(kondisi(suppressWarnings(as.numeric(n_i1)))),
                         as.character(input$ind_i_2), as.character(kondisi(suppressWarnings(as.numeric(n_i2)))),
                         as.character(nilai_i),
                         as.character(cond_i)))
      isolate(values_i$dfWorking <- as.data.frame(lapply(rbind(values_i$dfWorking,temp_df), as.character),stringsAsFactors=FALSE))
      isolate(names(values_i$dfWorking)<-c('Tahun', 
                                           'Kota.Kab', 
                                           'Nilai.I1', 'Kondisi.I1', 
                                           'Nilai.I2', 'Kondisi.I2',
                                           'Nilai.Total', 'Kondisi.Total'))
      #saveSQLite(values_i$dfWorking, "df_i")
      remove_i <-as.numeric({values_i$dfWorking[,1]})
      tahun_i_eliminate<- initialtahun[! initialtahun %in% remove_i]
      updateSelectInput(session, "thn_i", choices = tahun_i_eliminate)
      
    }
  })
  
  ### Menghapus Baris
  observeEvent(input$deleteRows_i,{
    
    if (!is.null(input$table_i_rows_selected)) {
      values_i$dfWorking <- values_i$dfWorking[-as.numeric(input$table_i_rows_selected),]
      #saveSQLite(values_i$dfWorking, "df_i")
      update_i <-as.numeric({values_i$dfWorking[,1]})
      tahun_i_update<- initialtahun[! initialtahun %in% update_i]
      updateSelectInput(session, "thn_i", choices = tahun_i_update)
    }
    else{
      values_i$dfWorking <- values_i$dfWorking[-as.numeric(nrow(values_i$dfWorking)),]
      #saveSQLite(values_i$dfWorking, "df_i")
      update_i <-as.numeric({values_i$dfWorking[,1]})
      tahun_i_update<- initialtahun[! initialtahun %in% update_i]
      updateSelectInput(session, "thn_i", choices = tahun_i_update)
    }
  })
  ### Mendownload File
  output$download_i <- downloadHandler(
    filename = function(){paste(format(Sys.time(), "%Y_%m_%d_"),"Indikator Lingkungan.csv", sep ="")}, 
    content = function(fname){
      file_i_unduh <- {values_i$dfWorking[order(values_i$dfWorking$Tahun, decreasing=F), ]}
      names(file_i_unduh) <- c('Tahun',
                               'Kota/Kab', 
                               'Nilai I1', 'Kondisi I1', 
                               'Nilai I2', 'Kondisi I2',
                               'Nilai Total', 'Kondisi Total')
      write.csv(file_i_unduh, fname, row.names = F)
    }
  )
  
  ### Untuk Upload Data
  filedata_i <- reactive({
    infile_i <- input$fileInput_i
    if (is.null(infile_i)) {
      # User has not uploaded a file yet
      return(NULL)
    }
    read.csv(infile_i$datapath, header=T, sep=",")
  })
  
  # Load File CSV untuk indikator Lingkungan
  observeEvent(input$shows_i,{
    if(!(is.null({filedata_i()}))){
      temp_i <- data.frame(isolate(filedata_i()))
      id_name_i <- c('Tahun',
                     'Kota.Kab', 
                     'Nilai.I1', 'Kondisi.I1', 
                     'Nilai.I2', 'Kondisi.I2',
                     'Nilai.Total', 'Kondisi.Total')
      if(identical(id_name_i, names(temp_i))){
        kon_i1<-apply(temp_i, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[3])), 0.5, 1, 2, 5))))
        new_df <- cbind(temp_i[1:nrow(temp_i), 1:3], "Kondisi.I1"=kon_i1, "Nilai.I2" = temp_i[1:nrow(temp_i), 5])
        kon_i2<-apply(temp_i, 1, function(x) kondisi(as.numeric(indikator_2a(suppressWarnings(as.numeric(x[5])), 0.5, 1, 3, 5))))
        new_df <- cbind(new_df, "Kondisi.I2"=kon_i2)
        
        ####
        result1 <- apply(temp_i, 1, 
                         function(x) mean(c(
                           indikator_2a(suppressWarnings(as.numeric(x[3])), 0.5, 1, 3, 5),
                           indikator_2a(suppressWarnings(as.numeric(x[5])), 0.5, 1, 3, 5)), na.rm = T
                         ))
        result1<-format(round(result1, 2), nsmall = 2)
        new_df <- cbind(new_df, "Nilai.Total"=result1)
        result2 <- apply(new_df, 1, function(x) kondisi(as.numeric(x[7])))
        new_df2 <- cbind(new_df, "Kondisi.Total"=result2)
        temp_i <- new_df2
        if(input$shows_i > 0){
          values_i$dfWorking <- as.data.frame(rbind(values_i$dfWorking,temp_i), stringsAsFactors=FALSE)
          values_i$dfWorking[] <- lapply(values_i$dfWorking, as.character)
          update_i <-as.numeric({values_i$dfWorking[,1]})
          tahun_i_update<- initialtahun[! initialtahun %in% update_i]
          updateSelectInput(session, "thn_i", choices = tahun_i_update) 
          temp_i <- NULL
        }else return (NULL)
      }
    }
  }
  )
  
  ################################################################
  #                                                              #
  #                    Indikator Lengkap                         #
  #                                                              #  
  ################################################################  
  
  ##
  output$tahun_lengkap_pilih<- renderUI({
    selectInput("thn_lengkap", "Tahun", as.list(initialtahun))
  })
  
  ##
  output$kota_lengkap_pilih<- renderUI({
    selectInput("kota_lengkap", "Kota/Kabupaten",
                choices = isolate(kota_pilih()))
  })
  
  ### Simpan Data
  observeEvent(input$save_lengkap,{
    if(input$save_lengkap > 0){
      df_lengkap_new<-values_lengkap$dfWorking
      df_lengkap_old<-subset(readSQLite('df_lengkap'), (Kota.Kab != City$Kota))
      if(City$Kota =="ALL"){
        lengkap_simpan$dfsimpan <- df_lengkap_new
      }else{
        lengkap_simpan$dfsimpan<-merge(df_lengkap_new, df_lengkap_old, all = T)}
      #print(lengkap_simpan$dfsimpan)
      saveSQLite(lengkap_simpan$dfsimpan, "df_lengkap")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data Indikator Lengkap berhasil!"
      ))
      saveSQLite(values_lengkap$dfWorking, "df_lengkap")
    }})
  
  
  ### Update Total Nilai dan KOndisi masing-masing Indikator
  observeEvent(input$get_data,{
    gettahun <- as.numeric(input$thn_lengkap)
    getkota <- input$kota_lengkap
    currtahun_e <- (values_e$dfWorking)
    currtahun_u <- (values_u$dfWorking)
    currtahun_l <- (values_l$dfWorking)
    currtahun_s <- (values_s$dfWorking)
    currtahun_i <- (values_i$dfWorking)
    if(input$get_data > 0 & !is.na(gettahun)){
      if(nrow(currtahun_e)!=0 & nrow(currtahun_u)!=0 & nrow(currtahun_l)!=0 & nrow(currtahun_s)!=0 & nrow(currtahun_i)!=0){
        e_nilai <- subset(currtahun_e["Nilai.Total"], 
                          currtahun_e["Tahun"] == gettahun & currtahun_e["Kota.Kab"] == getkota)
        e_kondisi <- subset(currtahun_e["Kondisi.Total"], 
                            currtahun_e["Tahun"] == gettahun & currtahun_e["Kota.Kab"] == getkota)
        u_nilai <- subset(currtahun_u["Nilai.Total"], 
                          currtahun_u["Tahun"] == gettahun & currtahun_u["Kota.Kab"] == getkota)
        u_kondisi <- subset(currtahun_u["Kondisi.Total"], 
                            currtahun_u["Tahun"] == gettahun & currtahun_u["Kota.Kab"] == getkota)
        l_nilai <- subset(currtahun_l["Nilai.Total"], 
                          currtahun_l["Tahun"] == gettahun & currtahun_l["Kota.Kab"] == getkota)
        l_kondisi <- subset(currtahun_l["Kondisi.Total"], 
                            currtahun_l["Tahun"] == gettahun & currtahun_l["Kota.Kab"] == getkota)
        s_nilai <- subset(currtahun_s["Nilai.Total"], 
                          currtahun_s["Tahun"] == gettahun & currtahun_s["Kota.Kab"] == getkota)
        s_kondisi <- subset(currtahun_s["Kondisi.Total"], 
                            currtahun_s["Tahun"] == gettahun & currtahun_s["Kota.Kab"] == getkota)
        i_nilai <- subset(currtahun_i["Nilai.Total"], 
                          currtahun_i["Tahun"] == gettahun & currtahun_i["Kota.Kab"] == getkota)
        i_kondisi <- subset(currtahun_i["Kondisi.Total"], 
                            currtahun_i["Tahun"] == gettahun & currtahun_i["Kota.Kab"] == getkota)
        if(nrow(e_nilai)!=0 & nrow(u_nilai)!=0 & nrow(l_nilai)!=0 & nrow(s_nilai)!=0 & nrow(i_nilai)!=0){
          lengkap_nilai <- suppressWarnings(mean(c(as.numeric(unlist(e_nilai)), as.numeric(unlist(u_nilai)), as.numeric(unlist(l_nilai)),
                                                   as.numeric(unlist(s_nilai)), as.numeric(unlist(i_nilai))), na.rm = T))
          lengkap_nilai <- suppressWarnings(format(round(lengkap_nilai, 2), nsmall = 2))
          
          lengkap_kon <- kondisi(suppressWarnings(as.numeric(lengkap_nilai)))
          #############################################################################
          temp_lengkap<-isolate(c(as.character(gettahun), as.character(getkota), 
                                  as.character(e_nilai), as.character(e_kondisi),
                                  as.character(u_nilai), as.character(u_kondisi),
                                  as.character(l_nilai), as.character(l_kondisi),
                                  as.character(s_nilai), as.character(s_kondisi),
                                  as.character(i_nilai), as.character(i_kondisi),
                                  as.character(lengkap_nilai),as.character(lengkap_kon)))
          
          isolate(values_lengkap$dfWorking <- as.data.frame(lapply(rbind(values_lengkap$dfWorking,temp_lengkap), 
                                                                   as.character),stringsAsFactors=FALSE))
          isolate(names(values_lengkap$dfWorking)<-c('Tahun', 
                                               'Kota.Kab', 
                                               'Ekonomi_nilai',
                                               'Ekonomi_kondisi',
                                               'Keuangan_nilai',
                                               'Keuangan_kondisi',
                                               'Kelembagaan_nilai',
                                               'Kelembagaan_kondisi',
                                               'Sosial_nilai',
                                               'Sosial_kondisi',
                                               'Lingkungan_nilai',
                                               'Lingkungan_kondisi',
                                               'Total_Nilai',
                                               'Total_Kondisi'))
          
          #saveSQLite(values_lengkap$dfWorking, "df_lengkap")
          remove_lengkap <-as.numeric({values_lengkap$dfWorking[,1]})
          tahun_lengkap_eliminate<- initialtahun[! initialtahun %in% remove_lengkap]
          updateSelectInput(session, "thn_lengkap", choices = tahun_lengkap_eliminate)
        }
      }
    }
  })
  
  ### Menghapus Baris
  observeEvent(input$clear_data,{
    
    if (!is.null(input$table_lengkap_rows_selected)) {
      values_lengkap$dfWorking <- values_lengkap$dfWorking[-as.numeric(input$table_lengkap_rows_selected),]
      #saveSQLite(values_lengkap$dfWorking, "df_lengkap")
      update_lengkap <-as.numeric({values_lengkap$dfWorking[,1]})
      tahun_lengkap_update<- initialtahun[! initialtahun %in% update_lengkap]
      updateSelectInput(session, "thn_lengkap", choices = tahun_lengkap_update)
    }
    else{
      values_lengkap$dfWorking <- values_lengkap$dfWorking[-as.numeric(nrow(values_lengkap$dfWorking)),]
      #saveSQLite(values_lengkap$dfWorking, "df_lengkap")
      update_lengkap <-as.numeric({values_lengkap$dfWorking[,1]})
      tahun_lengkap_update<- initialtahun[! initialtahun %in% update_lengkap]
      updateSelectInput(session, "thn_lengkap", choices = tahun_lengkap_update)
    }
  })
  
  ### Mendownload File
  output$download_lengkap <- downloadHandler(
    filename = function(){paste(format(Sys.time(), "%Y_%m_%d_"),"Indikator Monitoring Lengkap.csv", sep ="")}, 
    content = function(fname){
      file_lengkap_unduh <- {values_lengkap$dfWorking}
      names(file_lengkap_unduh) <- c('Tahun', 'Kota/Kab', 'Ekonomi_Nilai', 'Ekonomi_Kondisi', 'Keuangan_Nilai', 'Keuangan_Kondisi',
                                     'Kelembagaan_Nilai', 'Kelembagaan_Kondisi', 'Sosial_Nilai', 'Sosial_Kondisi',
                                     'Lingkungan_Nilai', 'Lingkungan_Kondisi',
                                     'Nilai Rataan Indikator', 'Kondisi Rataan Indikator')
      write.csv(file_lengkap_unduh, fname, row.names = F)
    }
  )
  
  #______________________________________________________________________________
  #observeEvent(session$isClosed()==T,{
    # print('byee')})
  # close the R session when Chrome closes
  #session$onSessionEnded(function() { 
   # print('byee') 
  #})
  
  ################################################################
  #                                                              #
  #                       User Organizer                         #
  #                                                              #  
  ################################################################  
  
  ## Menambah Baris 
  observeEvent(input$addRows_usr,{
    if(input$addRows_usr > 0){
      usr_df<-isolate(c(as.character(input$name_usr),
                        as.character(input$pass_usr),
                        as.character(input$kota_usr),
                        as.character(input$role_usr)
                        ))
      isolate(values_user_info$dfWorking <- as.data.frame(lapply(rbind(values_user_info$dfWorking,usr_df),
                                                                 as.character),stringsAsFactors=FALSE))
      isolate(names(values_user_info$dfWorking) <- c('user','pass','area','role'))
    }
  })
  
  ## Menghapus Baris
  observeEvent(input$deleteRows_usr,{
    if (!is.null(input$table_user_rows_selected)) {
      values_user_info$dfWorking <- values_user_info$dfWorking[-as.numeric(input$table_user_rows_selected),]
    }
    else{
      values_user_info$dfWorking <- values_user_info$dfWorking[-as.numeric(nrow(values_user_info$dfWorking)),]
    }
  })
  
  ## Menyimpan Data User
  observeEvent(input$save_user,{
    if(input$save_user > 0){
      temp_user<-values_user_info$dfWorking
      user_simpan$dfsafe <-temp_user
      saveSQLite(user_simpan$dfsafe, "membership")
      showModal(modalDialog(
        title = "Pemberitahuan",
        "Penyimpanan Data User berhasil!"
      ))
    }})
  
}