### Save database
saveSQLite <- function(data, name){
  dir.create(file.path(paste(dirname(getwd()),"/",basename(getwd()), sep=""), "basisdata"), showWarnings = FALSE)
  path <- paste(dirname(getwd()),"/",basename(getwd()),"/basisdata/",name,".sqlite", sep="")
  if (!file.exists(path)) {
    message("Menyimpan Basisdata pada direktori ", path)
    src <- src_sqlite(path, create = TRUE)
    copy_to(src, data, name, temporary = FALSE)
  } else {
    src <- src_sqlite(path)
    dbWriteTable(conn = src$con, name = name, 
                 value = data, row.names = FALSE, header = TRUE,
                 overwrite = T, append = F, temporary = FALSE)
  }
  return (src)
}

### Read database
readSQLite <- function(name){
  path <- paste(dirname(getwd()),"/",basename(getwd()),"/basisdata/",name,".sqlite", sep="")
  if (!file.exists(path)) {
    message("Basisdata tidak ditemukan pada direktori ", path)
  } else {
    con <- dbConnect(SQLite(), path)
    df<- dbReadTable(con, name)
  }
  return (df)
}


membership <- data.frame(
  user = c("deeboy","iboy", "dea", "tuti"),
  pass = c("geboy","aje", "amanda", "okeh"),
  area = c("ALL","BGR", "BGR", "MRK"),
  role = c("supadmin","manager", rep("user", 2)))
membership_db <- tbl(saveSQLite(membership, "membership"), "membership")

kota <- data.frame(
  abbrev = c("ALL","BGR", "MRK"),
  name = c("ALL","Kab. Bogor", "Kab. Merauke")
)
kota_db <- tbl(saveSQLite(kota, "kota"), "kota")

# Data Frame untuk Ekonomi
df_e <- data.frame(Tahun=as.Date(character()),
                   Kota.Kab=character(), 
                   Nilai.E1=character(),Kondisi.E1=character(),
                   Nilai.E2=character(),Kondisi.E2=character(),
                   Nilai.E3=character(),Kondisi.E3=character(),
                   Nilai.E4=character(),Kondisi.E4=character(),
                   Nilai.E5=character(),Kondisi.E5=character(),
                   Nilai.E6=character(),Kondisi.E6=character(),
                   Nilai.E7=character(),Kondisi.E7=character(),
                   Nilai.E8=character(),Kondisi.E8=character(),
                   Nilai.Total=character(),
                   Kondisi.Total=character(),
                   stringsAsFactors=FALSE)
ekonomi_db <- tbl(saveSQLite(df_e, "df_e"), "df_e")

# Data Frame untuk Keuangan
df_u <- data.frame(Tahun=as.Date(character()),
                   Kota.Kab=character(), 
                   Nilai.U1=character(),Kondisi.U1=character(),
                   Nilai.U2=character(),Kondisi.U2=character(),
                   Nilai.U3=character(),Kondisi.U3=character(),
                   Nilai.Total=character(),
                   Kondisi.Total=character(),
                   stringsAsFactors=FALSE)
keungan_db <- tbl(saveSQLite(df_u, "df_u"), "df_u")

# Data Frame untuk Kelembagaan
df_l <- data.frame(Tahun=as.Date(character()),
                   Kota.Kab=character(), 
                   Nilai.L1=character(),Kondisi.L1=character(), 
                   Nilai.L2=character(),Kondisi.L2=character(),
                   Nilai.L3=character(),Kondisi.L3=character(),
                   Nilai.Total=character(),
                   Kondisi.Total=character(),
                   stringsAsFactors=FALSE)
kelembagaan_db <- tbl(saveSQLite(df_l, "df_l"), "df_l")

# Data Frame untuk Sosial
df_s <- data.frame(Tahun=as.Date(character()),
                   Kota.Kab=character(), 
                   Nilai.S1=character(), Kondisi.S1=character(),
                   Nilai.S2=character(), Kondisi.S2=character(),
                   Nilai.S3=character(), Kondisi.S3=character(),
                   Nilai.S4=character(), Kondisi.S4=character(),
                   Nilai.S5=character(), Kondisi.S5=character(),
                   Nilai.Total=character(),
                   Kondisi.Total=character(),
                   stringsAsFactors=FALSE)
sosial_db <- tbl(saveSQLite(df_s, "df_s"), "df_s")

# Data Frame untuk Lingkungan
df_i <- data.frame(Tahun=as.Date(character()),
                   Kota.Kab=character(), 
                   Nilai.I1=character(), Kondisi.I1=character(),
                   Nilai.I2=character(), Kondisi.I2=character(),
                   Nilai=character(),
                   Kondisi=character(),
                   stringsAsFactors=FALSE)
lingkungan_db <- tbl(saveSQLite(df_i, "df_i"), "df_i")

# Data Frame untuk Tabel Lengkap
df_lengkap <- data.frame(Tahun=as.Date(character()),
                         Kota.Kab=character(), 
                         Ekonomi_nilai=character(),
                         Ekonomi_kondisi=character(),
                         Keuangan_nilai=character(),
                         Keuangan_kondisi=character(),
                         Kelembagaan_nilai=character(),
                         Kelembagaan_kondisi=character(),
                         Sosial_nilai=character(),
                         Sosial_kondisi=character(),
                         Lingkungan_nilai=character(),
                         Lingkungan_kondisi=character(),
                         Total_Nilai=character(),
                         Total_Kondisi=character(),
                         stringsAsFactors=FALSE)
lengkap_db <- tbl(saveSQLite(df_lengkap, "df_lengkap"), "df_lengkap")